/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry;

import java.util.Date;

/**
 * Represents comments and its metadata. Note that only the Comment.text field needs to be filled
 * when adding new comments. All other attributes are ignored and they are filled with appropriate
 * values for the current context. Therefore, when contructing an instance of this class outside
 * the Regisrty impl, it is recommended to use new Comment("my comment text") constructor.
 */
public class Comment {

    /**
     * Path of the comment. Each comment has a path in the form /projects/esb/config.xml;comments:12
     *
     */
    private transient String commentPath;

    /**
     * Comment text. This may contain any string inclusing HTML segments.
     */
    private transient String text;

    /**
     * Username of the user who added this comment.
     */
    private transient String user;

    /**
     * Date and time at which this comment is added.
     */
    private transient Date time;

    /**
     * Path of the resource on which this comment is made.
     */
    private transient String resourcePath;

    public Comment() {
    }

    public Comment(String commentText) {
        this.text = commentText;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public Date getTime() {
        return time;
    }

    public void setTime(Date time) {
        this.time = time;
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getCommentPath() {
        return commentPath;
    }

    public void setCommentPath(String commentPath) {
        this.commentPath = commentPath;
    }
}
