/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.protocol.server.ServiceContext;
import org.apache.abdera.protocol.server.impl.AbstractSingletonProviderManager;
import org.apache.abdera.protocol.server.impl.DefaultServiceContext;
import org.apache.abdera.protocol.server.servlet.AbderaServlet;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;
import org.wso2.registry.servlet.RegistryServlet;

/**
 * A very simple embeddable Registry server - handles the Registry wire protocol (APP) but has no UI
 * support built in.  Uses Jetty internally.
 */
public class RegistryServer {
    DefaultServiceContext abderaServiceContext = new RegistryAPPServiceContext();
    int port = 8081;
    String baseURL = "/wso2registry";
    //    Registry registry;
    Server server;

    public RegistryServer() {
    }

    public RegistryServer(int port) {
        this.port = port;
    }

    public RegistryServer(int port, String baseURL) {
        this.port = port;
        this.baseURL = baseURL;
    }

    public static void main(String[] args) throws Exception {
        RegistryServer server = new RegistryServer();
        server.start();
    }

    public void start() throws Exception {
        System.setProperty("in-memory", "true");
        RegistryAPPTargetResolver resolver = new RegistryAPPTargetResolver();
        abderaServiceContext.setTargetResolver(resolver);
        AbstractSingletonProviderManager pm = new RegistryAtomProviderManager();
        abderaServiceContext.setProviderManager(pm);

        server = new Server(port);
        Context root = new Context(server, baseURL, Context.SESSIONS);
        root.addServlet(new ServletHolder(new RegistryServlet() {
//            public void init() throws ServletException {
//                super.init();
//            }
        }), "/resources/*");
        root.addServlet(new ServletHolder(new AbderaServlet() {

            @Override
            protected ServiceContext createServiceContext() {
                abderaServiceContext
                        .setDefaultProviderManager(RegistryAtomProviderManager.class.getName());
                abderaServiceContext
                        .setDefaultTargetResolver(RegistryAPPTargetResolver.class.getName());
                abderaServiceContext.init(getAbdera(), getProperties(getServletConfig()));
                return abderaServiceContext;
            }

//            public void init() throws ServletException {
//                super.init();
//            }
        }), "/atom/*");
        server.start();
    }

    public void stop() throws Exception {
        if (server != null) server.stop();
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getBaseURL() {
        return baseURL;
    }

    public void setBaseURL(String baseURL) {
        this.baseURL = baseURL;
    }
}
