/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory;

import org.wso2.registry.RegistryConstants;

import java.util.HashMap;

/**
 * This Map prepares all paths used as keys by removing suffixing "/" charactors. So that the
 * registry does not distinguesh paths ending with "/" and those do not. i.e. /d1/d2 is considered
 * equal to /d1/d2/
 */
public class ResourceMap extends HashMap {

    public Object put(Object key, Object value) {
        return super.put(preparePath((String)key), value);
    }

    public Object get(Object key) {
        return super.get(preparePath((String)key));
    }

    public boolean containsKey(Object key) {
        return super.containsKey(preparePath((String)key));
    }

    public Object remove(Object key) {
        return super.remove(preparePath((String)key));
    }

    private String preparePath(String path) {
        String preparedPath = path;
        // make sure that the path does not end with a "/"
        if (!path.equals(RegistryConstants.ROOT_PATH) &&
            path.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            preparedPath =
                    path.substring(0, path.length() - RegistryConstants.PATH_SEPARATOR.length());
        }
        return preparedPath;
    }
}
