/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory.comments;

import org.wso2.registry.Comment;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.inmemory.InMemoryRegistry;

import java.util.*;

public class CommentManager {

    private InMemoryRegistry inMemoryRegistryProvider = null;

    private Map comments = new HashMap();

    public CommentManager(InMemoryRegistry inMemoryRegistryProvider) {
        this.inMemoryRegistryProvider = inMemoryRegistryProvider;
    }

    public int addComment(String resourcePath, Comment comment) throws RegistryException {

        if (!inMemoryRegistryProvider.isResourceAvailable(resourcePath)) {
            throw new RegistryException(Messages.getMessage("cannotadd.comment", resourcePath));
        }

        comment.setTime(new Date());

        List resourceComments = (List)comments.get(resourcePath);

        if (resourceComments == null) {
            resourceComments = new ArrayList();
            comments.put(resourcePath, resourceComments);
        }

        resourceComments.add(comment);
        return resourceComments.size() - 1;
    }

    public Comment[] getComments(String resourcePath) {

        List resourceComments = (List)comments.get(resourcePath);

        if (resourceComments == null) {
            return new Comment[0];
        }

        return (Comment[])resourceComments.toArray(new Comment[resourceComments.size()]);
    }

    public void removeComments(String resourcePath) {

        comments.remove(resourcePath);
    }
}
