/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.inmemory.ratings;

import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.inmemory.InMemoryRegistry;

import java.util.HashMap;
import java.util.Map;

public class RatingsManager {

    private InMemoryRegistry inMemoryRegistryProvider = null;

    /** Map containing resourcePath --> Rating */
    private Map ratings = new HashMap();

    public RatingsManager(InMemoryRegistry inMemoryRegistryProvider) {
        this.inMemoryRegistryProvider = inMemoryRegistryProvider;
    }

    public void rateResource(String resourcePath, int rating) throws RegistryException {

        if (!inMemoryRegistryProvider.isResourceAvailable(resourcePath)) {
            throw new RegistryException(Messages.getMessage("cannot.rate", resourcePath));
        }

        ResourceRating resourceRating = (ResourceRating)ratings.get(resourcePath);
        if (resourceRating == null) {
            resourceRating = new ResourceRating();
            ratings.put(resourcePath, resourceRating);
        }

        resourceRating.addRating(rating);
    }

    public float getAverageRating(String resourcePath) {

        float averageRating = 0;

        ResourceRating resourceRating = (ResourceRating)ratings.get(resourcePath);
        if (resourceRating != null) {
            averageRating = resourceRating.getAverageRating();
        }

        return averageRating;
    }

    public void removeRatings(String resourcePath) {
        ratings.remove(resourcePath);
    }
}
