/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.hsql;

import java.sql.*;

public class DBUtils {

    private static final String artifactsTable =
            "CREATE TABLE ARTIFACTS (AID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "PATH VARCHAR (500) NOT NULL," +
            "MEDIA_TYPE VARCHAR (500)," +
            "DIRECTORY INTEGER NOT NULL," +
            "STATE INTEGER," +
            "AUTHOR VARCHAR (500)," +
            "CREATED_TIME TIMESTAMP," +
            "DESCRIPTION VARCHAR (500)," +
            "PRIMARY KEY (AID)," +
            "UNIQUE(PATH))";

    private static final String propertiesTable =
            "CREATE TABLE PROPERTIES (P_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "AID INTEGER NOT NULL," +
            "PKEY VARCHAR (100) NOT NULL," +
            "PVALUE VARCHAR (500) NOT NULL," +
            "PRIMARY KEY (P_ID)," +
            "UNIQUE (AID, PKEY)," +
            "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID))";

    private static final String childrenTable =
            "CREATE TABLE CHILDREN (CID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "PARENT_ID INTEGER NOT NULL," +
            "CHILD_ID INTEGER NOT NULL," +
            "PRIMARY KEY (CID)," +
            "UNIQUE (PARENT_ID, CHILD_ID)," +
            "FOREIGN KEY (PARENT_ID) REFERENCES ARTIFACTS (AID)," +
            "FOREIGN KEY (CHILD_ID) REFERENCES ARTIFACTS (AID))";

    private static final String tagsTable =
            "CREATE TABLE TAGS (TN_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "TAG_NAME VARCHAR (500) NOT NULL," +
            "AID INTEGER NOT NULL," +
            "USER_ID VARCHAR (20) NOT NULL," +
            "TAGGED_TIME TIMESTAMP NOT NULL," +
            "PRIMARY KEY (TN_ID)," +
            "UNIQUE (TAG_NAME, AID, USER_ID)," +
            "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID))";

    private static final String commentsTable =
            "CREATE TABLE COMMENTS (CM_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "AID INTEGER NOT NULL," +
            "USER_ID VARCHAR (20) NOT NULL," +
            "COMMENT_TEXT VARCHAR (500) NOT NULL," +
            "COMMENTED_TIME TIMESTAMP NOT NULL," +
            "PRIMARY KEY (CM_ID)," +
            "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID))";

    private static final String logsTable =
            "CREATE TABLE LOGS (LOG_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "RID INTEGER," +
            "USER_ID VARCHAR (20) NOT NULL," +
            "LOGGED_TIME TIMESTAMP NOT NULL," +
            "ACTION INTEGER NOT NULL," +
            "ACTION_DATA VARCHAR (500)," +
            "PRIMARY KEY (LOG_ID))";

    private static final String ratingsTable =
            "CREATE TABLE RATINGS (R_ID INTEGER GENERATED BY DEFAULT AS IDENTITY," +
            "AID INTEGER NOT NULL," +
            "USER_ID VARCHAR (20) NOT NULL," +
            "RATING INTEGER NOT NULL," +
            "RATED_TIME TIMESTAMP NOT NULL," +
            "PRIMARY KEY (R_ID)," +
            "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID))";

    private static final String versionTable = "CREATE TABLE VERSIONS (" +
                                               "AID INTEGER NOT NULL," +
                                               "VN INTEGER NOT NULL," +
                                               "CONTENT BINARY, " +
                                               "AUTHOR VARCHAR (500), " +
                                               "UPDATED_TIME TIMESTAMP," +
                                               "UNIQUE (AID,VN)," +
                                               "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));";

    private static final String dependencyTable = "CREATE TABLE DEPENDENCY (" +
                                                  "AID INTEGER NOT NULL," +
                                                  "VN INTEGER NOT NULL," +
                                                  "DAID INTEGER NOT NULL," +
                                                  "DVN INTEGER NOT NULL," +
                                                  "UNIQUE (AID,VN,DAID,DVN)," +
                                                  "FOREIGN KEY (DAID) REFERENCES ARTIFACTS (AID)," +
                                                  "FOREIGN KEY (AID) REFERENCES ARTIFACTS (AID));";

    public void initializeDatabase(Connection conn) throws SQLException {

        DatabaseMetaData metaData = conn.getMetaData();
        ResultSet result = metaData.getTables(null, null, "artifacts", null);
        if (result.next()) {
            return;
        }

        ResultSet result2 = metaData.getTables(null, null, "ARTIFACTS", null);
        if (result2.next()) {
            return;
        }

        Statement s = conn.createStatement();
        s.executeUpdate(artifactsTable);
        s.executeUpdate(propertiesTable);
        s.executeUpdate(childrenTable);
        s.executeUpdate(tagsTable);
        s.executeUpdate(commentsTable);
        s.executeUpdate(ratingsTable);
        s.executeUpdate(logsTable);

        // creating version related tables
        s.executeUpdate(versionTable);
        s.executeUpdate(dependencyTable);
    }
}
