/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin;

import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.mediatypes.MediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;

public class SQLQueryMediaTypeHandler extends MediaTypeHandler {

    public SQLQueryMediaTypeHandler(DataSource dataSource,
                                    Realm realm,
                                    MediaTypeManager mediaTypeManager) {

        super(dataSource, realm, mediaTypeManager);
    }

    public Resource get(String path, Resource resource) throws RegistryException {

        Object content = resource.getContent();
        if (content instanceof byte[]) {
            resource.setContent(new String((byte[])content));
        }

        return resource;
    }

    public boolean put(String path, Resource resource) throws RegistryException {

        Object content = resource.getContent();
        if (content instanceof String) {
            String textContent = (String)content;
            resource.setContent(textContent.getBytes());
        }

        MediaTypeHandler defaultMediaTypeHandler = mediaTypeManager.getDefaultMediaTypeHandler();
        defaultMediaTypeHandler.put(path, resource);

        return true;
    }

    public String importResource(String path, String sourceURL, Resource metadata)
            throws RegistryException {
        return null;
    }

    public boolean delete(String path) throws RegistryException {
        // we are not doing anything special for deleting sql queries
        return false;
    }

    public boolean putChild(String childPath, Resource resource) throws RegistryException {
        // SQL query is a non-collection resource. So don't allow any children.
        return false;
    }

    public boolean importChild(String childPath, String sourceURL) throws RegistryException {
        return false;
    }
}
