/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.mediatypes.MediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.registry.jdbc.mediatypes.utils.SchemaFileProcessor;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;

public class XSDMediaTypeHandler extends MediaTypeHandler {

    SchemaFileProcessor schemaFileProcessor;

    public XSDMediaTypeHandler(DataSource dataSource,
                               Realm realm,
                               MediaTypeManager mediaTypeManager) {

        super(dataSource, realm, mediaTypeManager);

        schemaFileProcessor =
                new SchemaFileProcessor(mediaTypeManager.getDefaultMediaTypeHandler());

    }

    public Resource get(String path, Resource rawArtifact) throws RegistryException {
        return null;
    }

    public boolean put(String path, Resource resource) throws RegistryException {
        // let the default media type handler handle if the xsd is given as a resource. because
        // we can't import dependencies, if it is given as a resource.
        return false;
    }

    public String importResource(String path, String sourceURL, Resource metadata)
            throws RegistryException {

        String savedName = schemaFileProcessor
                .saveSchemaFileToRegistry(sourceURL, getParentPath(path), true, metadata);

        String parentPath = AuthorizationUtil.getParentPath(path);

        if (parentPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            return parentPath + savedName;
        } else {
            return parentPath + RegistryConstants.PATH_SEPARATOR + savedName;
        }
    }

    public boolean delete(String path) throws RegistryException {
        return false;
    }

    public boolean putChild(String childPath, Resource resource) throws RegistryException {
        return false;
    }

    public boolean importChild(String childPath, String sourceURL) throws RegistryException {
        return false;
    }

    private String getParentPath(String childPath) {

        if (childPath.equals(RegistryConstants.ROOT_PATH)) {
            return null;
        }

        int parentPathLength = childPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR);

        if (parentPathLength == 0) {
            return RegistryConstants.ROOT_PATH;
        }

        return childPath.substring(0, parentPathLength);
    }
}
