/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.wso2.registry.RegistryConstants;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultAccessControlAdmin;

import javax.sql.DataSource;

public class RegistryAccessControlAdmin extends DefaultAccessControlAdmin {

    public RegistryAccessControlAdmin(DataSource dataSource, String algo) {
        super(dataSource, algo);
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserManagerException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserManagerException(msg, false);
        }

        super.clearUserAuthorization(userName, resourceId, action);
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserManagerException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserManagerException(msg, false);
        }

        super.denyUser(userName, resourceId, action);
    }


    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserManagerException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserManagerException(msg, false);
        }

        super.clearRoleAuthorization(roleName, resourceId, action);
    }


    public void denyRole(String roleName, String resourceId, String action)
            throws UserManagerException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserManagerException(msg, false);
        }

        super.denyRole(roleName, resourceId, action);
    }
}
