/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.wso2.registry.RegistryException;
import org.wso2.registry.utils.AuthorizationUtil;
import org.wso2.usermanager.AccessControlAdmin;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.UserStoreAdmin;
import org.wso2.usermanager.readwrite.DefaultRealm;
import org.wso2.usermanager.readwrite.DefaultRealmConfig;

import javax.sql.DataSource;

public class RegistryRealm extends DefaultRealm {

    private RegistryUserStoreAdmin userStoreAdmin = null;
    private RegistryAccessControlAdmin accessControlAdmin = null;

    public RegistryRealm(DataSource dataSource) throws RegistryException {
        super();
        init(dataSource);
    }

    protected RegistryRealm() throws RegistryException {
    }

    protected void configure(DataSource dataSource) throws RegistryException {
        init(dataSource);
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserManagerException {
        return userStoreAdmin;
    }

    public AccessControlAdmin getAccessControlAdmin() throws UserManagerException {
        return accessControlAdmin;
    }

    private void init(DataSource dataSource) throws RegistryException {

        try {
            DefaultRealmConfig config = (DefaultRealmConfig)this.getRealmConfiguration();
            config.setDataSource(dataSource);
            this.init(config);

            userStoreAdmin = new RegistryUserStoreAdmin(dataSource);
            accessControlAdmin = new RegistryAccessControlAdmin(
                    dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST);

            AuthorizationUtil.populateUserStore(this);

        } catch (UserManagerException e) {
            String msg = "Could not initiate a user manager realm using the given data source. " +
                         "Caused by: " + e.getMessage();
            throw new RegistryException(msg, e);
        }
    }
}
