/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.wso2.registry.RegistryConstants;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultUserStoreAdmin;

import javax.sql.DataSource;

public class RegistryUserStoreAdmin extends DefaultUserStoreAdmin {

    public RegistryUserStoreAdmin(DataSource dataSource) throws UserManagerException {
        super(dataSource);
    }

    public void addUser(String userName, Object credential) throws UserManagerException {
        super.addUser(userName, credential);
        addUserToRole(userName, RegistryConstants.EVERYONE_ROLE);
    }

    public void deleteUser(String userName) throws UserManagerException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER) ||
            userName.equals(RegistryConstants.ANONYMOUS_USER)) {

            String msg = "Could not remove the system defined user: " + userName;
            throw new UserManagerException(msg, false);
        }

        super.deleteUser(userName);
    }


    public void deleteRole(String roleName) throws UserManagerException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE) ||
            roleName.equals(RegistryConstants.EVERYONE_ROLE) ||
            roleName.equals(RegistryConstants.GUESTS_ROLE)) {

            String msg = "Could not remove the system defined role: " + roleName;
            throw new UserManagerException(msg, false);
        }

        super.deleteRole(roleName);
    }


    public void removeUserFromRole(String userName, String roleName) throws UserManagerException {

        if (roleName.equals(RegistryConstants.EVERYONE_ROLE)) {

            String msg = "Could not remove the everyone role from the user: " +
                         userName + ". All users belong to the everyone role.";
            throw new UserManagerException(msg, false);
        }

        super.removeUserFromRole(userName, roleName);
    }


    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserManagerException {

        // we should not allow to edit anoymous or system users.
        if (RegistryConstants.SYSTEM_USER.equals(userName) || 
                RegistryConstants.ANONYMOUS_USER.equals(userName)) {

            String msg = "Could not edit the system defined user: " + userName;
            throw new UserManagerException(msg);
        }

        super.updateUser(userName, newCredential, oldCredential);
    }
}
