/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.dao.RatingsDAO;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Date;

public class RatingURLHandler extends URLHandler {

    private static final Log log = LogFactory.getLog(RatingURLHandler.class);

    public RatingURLHandler(DataSource dataSource, Realm realm) {
        super(dataSource, realm);
    }

    public Resource get(String url) throws RegistryException {

        String[] parts = url.split(";");

        if (parts.length != 2) {
            return null;
        }

        if (parts[0].length() == 0) {
            return null;
        }

        String[] details = parts[1].split(":");
        if (details.length != 2 || !details[0].equals("ratings") || details[1].length() == 0) {
            return null;
        }

        String resourcePath = parts[0];
        String ratedUserName = details[1];

        Connection conn;

        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            RatingsDAO ratingsDAO = new RatingsDAO();
            int rating = ratingsDAO.getRating(resourcePath, ratedUserName, conn);
            Date ratedTime = ratingsDAO.getRatedTime(resourcePath, ratedUserName, conn);

            Resource resource = new Resource();
            resource.setMediaType(RegistryConstants.RATING_MEDIA_TYPE);
            resource.setContent(rating);
            resource.setAuthorUserName(ratedUserName);
            resource.setPath(url);
            resource.setCreatedTime(ratedTime);
            resource.setProperty("resourcePath", resourcePath);

            return resource;

        } catch (SQLException e) {
            String msg = "Could not get the rating for resource at " + resourcePath +
                         ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException ignore) {
                    log.info("Error while closing the DB connection");
                }
            }
        }
    }
}
