/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.urlhandlers;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.dao.RatingsDAO;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class RatingsCollectionURLHandler extends URLHandler {

    private static final Log log = LogFactory.getLog(CommentCollectionURLHandler.class);

    public RatingsCollectionURLHandler(DataSource dataSource, Realm realm) {
        super(dataSource, realm);
    }

    public Resource get(String url) throws RegistryException {

        String[] parts = url.split(";");

        if (parts.length != 2) {
            return null;
        }

        if (parts[0].length() == 0) {
            return null;
        }

        if (!parts[1].equals("ratings")) {
            return null;
        }

        if (parts[1].length() != "ratings".length()) {
            return null;
        }

        String resourcePath = parts[0];

        Connection conn;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        RatingsDAO ratingsDAO = new RatingsDAO();

        try {
            Resource resource = new Resource();
            resource.setPath(url);
            resource.setDirectory(true);

            String[] ratedUserNames = ratingsDAO.getRatedUserNames(resourcePath, conn);
            List ratingPaths = new ArrayList();
            for (String ratedUserName : ratedUserNames) {
                String ratingPath = resourcePath + ";ratings:" + ratedUserName;
                ratingPaths.add(ratingPath);
            }

            String[] ratingsContent = (String[])ratingPaths.toArray(new String[ratingPaths.size()]);
            resource.setContent(ratingsContent);

            return resource;

        } catch (SQLException e) {
            String msg = "Failed to get ratings for resource: " +
                         resourcePath + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info("exception.closing.db");
            }
        }
    }
}
