/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryException;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.util.DefaultDatabaseUtil;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

public class HSQLDBInitializer {

    private static final Log log = LogFactory.getLog(HSQLDBInitializer.class);

    public void createHSQLTables(DataSource dataSource) throws RegistryException {

        Connection conn;
        try {
            conn = dataSource.getConnection();

        } catch (SQLException e) {
            String msg = "Could not get the HSQL DB connection for creating tables.";
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }

        try {
            DatabaseMetaData metaData = conn.getMetaData();
            ResultSet result = metaData.getTables(null, null, "um_users", null);
            if (result.next()) {
                log.info("User Manager tables were already created.");
                return;
            }

            ResultSet result2 = metaData.getTables(null, null, "UM_USERS", null);
            if (result2.next()) {
                log.info("User Manager tables were already created.");
                return;
            }

        } catch (SQLException e) {
            String msg = "Could not get the metadata of the User Manager database.";
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        }

        try {

            DefaultDatabaseUtil.createDatabase(conn);

        } catch (UserManagerException e) {
            String msg = "Could not create tables for user manager";
            log.fatal(msg, e);
            throw new RegistryException(msg, e);
        } catch (SQLException e) {
            String msg = "Could not create tables for the User Manager.";
            log.fatal(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            try {
                conn.close();
            } catch (SQLException ignore) {
                log.info(ignore);
            }
        }
    }
}
