/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.servlet.utils;

import org.wso2.registry.ActionConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerConstants;
import org.wso2.usermanager.UserManagerException;

import javax.servlet.http.HttpServletRequest;

public class UserUtil {

    public static boolean isPutAllowed(
            String userName, String resourcePath, HttpServletRequest request)
            throws RegistryException {

        boolean putAllowed = false;

        SecureRegistry secureRegistry = Utils.getSecureRegistry(request);
        Realm userRealm = secureRegistry.getUserRealm();

        try {
            if (userRealm.getAuthorizer().isUserAuthorized(
                    userName, resourcePath, ActionConstants.PUT)) {
                putAllowed = true;
            }
        } catch (UserManagerException e) {

            String msg = "Could not the permission details for the user: " + userName +
                         " for the resource: " + resourcePath + ". Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return putAllowed;
    }

    public static boolean isDeleteAllowed(
            String userName, String resourcePath, HttpServletRequest request)
            throws RegistryException {

        boolean putAllowed = false;

        SecureRegistry secureRegistry = Utils.getSecureRegistry(request);
        Realm userRealm = secureRegistry.getUserRealm();

        try {
            if (userRealm.getAuthorizer().isUserAuthorized(
                    userName, resourcePath, ActionConstants.DELETE)) {
                putAllowed = true;
            }
        } catch (UserManagerException e) {

            String msg = "Could not the permission details for the user: " + userName +
                         " for the resource: " + resourcePath + ". Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return putAllowed;
    }

    public static boolean isAuthorizeAllowed(
            String userName, String resourcePath, HttpServletRequest request)
            throws RegistryException {

        boolean putAllowed = false;

        SecureRegistry secureRegistry = Utils.getSecureRegistry(request);
        Realm userRealm = secureRegistry.getUserRealm();

        try {
            if (userRealm.getAuthorizer().isUserAuthorized(
                    userName, resourcePath, UserManagerConstants.AUTHORIZE)) {
                putAllowed = true;
            }
        } catch (UserManagerException e) {

            String msg = "Could not the permission details for the user: " + userName +
                         " for the resource: " + resourcePath + ". Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return putAllowed;
    }
}
