/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.utils;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.usermanager.*;

public class AuthorizationUtil {

    private static final Log log = LogFactory.getLog(AuthorizationUtil.class);

    public static void setDefaultAuthorizations(Realm realm, String path, String authorUserName)
            throws RegistryException {

        if (realm == null) {
            return;
        }

        try {
            AccessControlAdmin accessControlAdmin = realm.getAccessControlAdmin();

            if (path.equals(RegistryConstants.ROOT_PATH)) {

                accessControlAdmin
                        .authorizeUser(RegistryConstants.SYSTEM_USER, path, ActionConstants.GET);
                accessControlAdmin
                        .authorizeUser(RegistryConstants.SYSTEM_USER, path, ActionConstants.PUT);
                accessControlAdmin
                        .authorizeUser(RegistryConstants.SYSTEM_USER, path, ActionConstants.DELETE);
                accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER, path,
                                                 UserManagerConstants.AUTHORIZE);

                accessControlAdmin
                        .authorizeUser(RegistryConstants.ADMIN_USER, path, ActionConstants.GET);
                accessControlAdmin
                        .authorizeUser(RegistryConstants.ADMIN_USER, path, ActionConstants.PUT);
                accessControlAdmin
                        .authorizeUser(RegistryConstants.ADMIN_USER, path, ActionConstants.DELETE);
                accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER, path,
                                                 UserManagerConstants.AUTHORIZE);

                accessControlAdmin
                        .authorizeRole(RegistryConstants.ADMIN_ROLE, path, ActionConstants.GET);
                accessControlAdmin
                        .authorizeRole(RegistryConstants.ADMIN_ROLE, path, ActionConstants.PUT);
                accessControlAdmin
                        .authorizeRole(RegistryConstants.ADMIN_ROLE, path, ActionConstants.DELETE);
                accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE, path,
                                                 UserManagerConstants.AUTHORIZE);

                accessControlAdmin
                        .authorizeRole(RegistryConstants.EVERYONE_ROLE, path, ActionConstants.GET);

            } else {

                String parentPath = getParentPath(path);
                accessControlAdmin.copyAuthorizations(parentPath, path);

                accessControlAdmin.authorizeUser(authorUserName, path, ActionConstants.GET);
                accessControlAdmin.authorizeUser(authorUserName, path, ActionConstants.PUT);
                accessControlAdmin.authorizeUser(authorUserName, path, ActionConstants.DELETE);
                accessControlAdmin
                        .authorizeUser(authorUserName, path, UserManagerConstants.AUTHORIZE);
            }

        } catch (UserManagerException e) {
            String msg = "Could not default permissions. \nCaused by: " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg);
        }
    }

    public static void populateUserStore(Realm realm) throws UserManagerException {

        if (realm == null) {
            return;
        }

        UserStoreReader userStoreReader = realm.getUserStoreReader();
        UserStoreAdmin userStoreAdmin = realm.getUserStoreAdmin();
        AccessControlAdmin accessControlAdmin = realm.getAccessControlAdmin();

        String[] roles = userStoreReader.getAllRoleNames();

        if (!containsString(RegistryConstants.ADMIN_ROLE, roles)) {

            userStoreAdmin.addRole(RegistryConstants.ADMIN_ROLE);

            // authorizations for user management
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeRole(RegistryConstants.ADMIN_ROLE,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.DELETE);
        }

        if (!containsString(RegistryConstants.GUESTS_ROLE, roles)) {
            userStoreAdmin.addRole(RegistryConstants.GUESTS_ROLE);
        }

        if (!containsString(RegistryConstants.EVERYONE_ROLE, roles)) {
            userStoreAdmin.addRole(RegistryConstants.EVERYONE_ROLE);

            accessControlAdmin.authorizeRole(RegistryConstants.EVERYONE_ROLE,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeRole(RegistryConstants.EVERYONE_ROLE,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeRole(RegistryConstants.EVERYONE_ROLE,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.READ);
        }

        // define built in users

        String[] users = userStoreReader.getAllUserNames();

        if (!containsString(RegistryConstants.SYSTEM_USER, users)) {

            userStoreAdmin
                    .addUser(RegistryConstants.SYSTEM_USER, RegistryConstants.SYSTEM_PASSWORD);

            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.SYSTEM_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.DELETE);
        }

        if (!containsString(RegistryConstants.ADMIN_USER, users)) {

            userStoreAdmin.addUser(RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);

            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.ROLE_RESOURCE,
                                             UserManagerConstants.DELETE);

            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.READ);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.ADD);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.EDIT);
            accessControlAdmin.authorizeUser(RegistryConstants.ADMIN_USER,
                                             UserManagerConstants.USER_PERMISSION_RESOURCE,
                                             UserManagerConstants.DELETE);
        }

        if (!containsString(RegistryConstants.ANONYMOUS_USER, users)) {
            userStoreAdmin.addUser(RegistryConstants.ANONYMOUS_USER,
                                   RegistryConstants.ANONYMOUS_PASSWORD);
        }
    }

    public static String getParentPath(String childPath) {

        if (childPath.equals(RegistryConstants.ROOT_PATH)) {
            return null;
        }

        int parentPathLength = childPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR);

        if (parentPathLength == 0) {
            return RegistryConstants.ROOT_PATH;
        }

        return childPath.substring(0, parentPathLength);
    }

    /**
     * Path of a resource given to the Registry interface may contain extensions to refer metadata
     * about resources. But we always store the authorization for resources against the pure
     * resource path, stored in the ARTIFACTS table. This methods extracts that pure resource path
     * from a given path.
     *
     * @param resourcePath A path string, which may contain extensions
     * @return pure resource path for the given path
     */
    public static String getAuthorizationPath(String resourcePath) {

        // if the user has permission to the current version, he will get permission to all
        // previous versions of the same resource

        String preparedPath = resourcePath;
        if (resourcePath.indexOf("?") > 0) {
            preparedPath = resourcePath.split("\\?")[0];
        } else if (resourcePath.indexOf(";") > 0) {
            preparedPath = resourcePath.split("\\;")[0];
        }

        if (preparedPath.equals(RegistryConstants.ROOT_PATH)) {
            return preparedPath;

        } else {

            if (!preparedPath.startsWith(RegistryConstants.ROOT_PATH)) {
                preparedPath = RegistryConstants.ROOT_PATH + preparedPath;
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(0, preparedPath.length() - 1);
            }
        }

        return preparedPath;
    }

    private static boolean containsString(String value, String[] array) {
        for (int i = 0; i < array.length; i++) {
            if (value.equals(array[i])) {
                return true;
            }
        }
        return false;
    }
}
