/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.wso2.registry.RegistryConstants;

/**
 * These tests use a raw HTTP client in order to exercise the APP implementation of our Registry
 * server.  We do this to make sure that the protocol exactly matches what we expect; using the
 * RemoteRegistry (as the JettyBasedServerTest does) is a "black box" test which only proves the API
 * works.  This test confirms wire-level behavior.
 * <p/>
 * TODO: WORK IN PROGRESS (Glen)
 */
public class APPProtocolTest extends AbstractAPPTest   {
    HttpClient client = new HttpClient();

    public String buildURL(String path) {
        return BASEURL + path;
    }

    /**
     * Run through a series of actions against a clean registry, and confirm the results match our
     * expectations.
     *
     * @throws Exception if there's a problem
     */
    public void testProtocol() throws Exception {
        // Note - every URL below is rooted at our Registry's atom root

        // GET the root - empty dir
        HttpMethod method = new GetMethod(buildURL(""));
        int responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);

        method.getResponseBody();

        // GET /d1 - 404
        method = new GetMethod(buildURL("d1"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode); // Nothing there yet!

        // GET /foo;tags - 404
        // Make sure metadata indicator doesn't change error code
        method = new GetMethod(buildURL("foo" + RegistryConstants.URL_SEPARATOR + "tags"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode);

/*
        // POST / with d1 content
        // TODO - fix this up.
        String atomContent =
                "<entry xmlns=\"http://www.w3.org/2005/Atom\" xmlns:ns=\"tag:wso2.org,2008:foo\"><summary type=\"text\" /><author><name>system</name></author><ns:properties /><mediaType xmlns=\"http://wso2.org/registry\" /><parentPath xmlns=\"http://wso2.org/registry\" /><directory xmlns=\"http://wso2.org/registry\">true</directory></entry>";
        String location = doPost(buildURL(""), atomContent, "d1");
        assertNotNull(location);
        //assertEquals("correct header value", location);

        // GET /d1 - empty dir (but not 404!)
        method = new GetMethod(buildURL("d1"));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);

        // GET /d1;tags - empty tags (or 404?)
        method = new GetMethod(buildURL("d1;tags"));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);

        // POST / with no slug - text file
        String textContent = "hi there here's some text";
        location = doPost("", textContent, null);
        assertNotNull(location);

        // GET /<newname> -
        method = new GetMethod(buildURL(location));
        responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);
        // We should have here an Atom entry pointing to the actual media.  Parse
        // that out and get the media URL, then GET that and make sure the content
        // is kosher.

        // POST /d1 with bad slug "&test?"
        // Should return /d1/test (sanitized name)
        //assertEquals("d1/test", location);

        // GET /d1/test - content

        // POST /d1 with existing slug "test"
        // Should return /d1/test2 (edited name)
        assertFalse("test".equals(location));

        // GET /d1/test2 - content

        // Tags

        // GET /d1/test;tags - empty tags (or 404?)

        // POST /d1/test;tags tag=blue

        // GET /d1/test;tags - "blue"

        // Ratings

        // Comments
*/
    }

    private String doPost(String url, String content, String slug) throws Exception {
        PostMethod writeMethod = new PostMethod(url);
        if (slug != null)
            writeMethod.addRequestHeader("Slug", slug);
        StringRequestEntity myRequest = new StringRequestEntity(content);
        writeMethod.setRequestEntity(myRequest);
        int responseCode = client.executeMethod(writeMethod);
        assertEquals(200, responseCode);

        // Get the location header
        String location = writeMethod.getResponseHeader("Location").getValue();
        assertNotNull(location);

        // Parse out result path
        return location;
    }
}
