/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.app;

import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;

import java.util.Properties;

public class UserTest extends AbstractAPPTest {
    public void testHierachicalResource() throws Exception {
        Resource r1 = new Resource();
        String content = "this is my content1";
        r1.setContent(content.getBytes());
        r1.setDescription("This is r1 file description");
        r1.setDirectory(false);

        String path = "/d1/d2/d3/r1";

        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /d1/d2/d3/r1");
        }

        Resource r1_actual = new Resource();

        try {
            r1_actual = registry.get("/d1/d2/d3/r1");
        } catch (RegistryException e) {
            fail("Couldn't get content from path /d1/d2/d3/r1");
        }

//         //System.out.println("File content:" + new String((byte[]) r1.getContent()));
//         //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//         //System.out.println("Path:" + r1_actual.getResourcePath());
//         //System.out.println("File Desciption:" + r1_actual.getDescription());
//         //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//         //System.out.println("Resource Properties:" + r1_actual.getProperties());
//         //System.out.println("Resource is Directory:" + r1_actual.isDirectory());

        assertEquals("Content is not equal.", new String((byte[])r1.getContent()),
                     new String((byte[])r1_actual.getContent()));
        assertEquals("LastUpdatedUser is not Equal", "admin", r1_actual.getLastUpdaterUserName());

    }


    public void testUpdateResouceContent() throws Exception {

        Resource r1 = new Resource();
        String content = "this is my content1";
        r1.setContent(content.getBytes());
        r1.setDescription("This is r1 file description");
        r1.setDirectory(false);

        Properties r2Pro = new Properties();
        r2Pro.setProperty("key1", "value1");
        r2Pro.setProperty("key2", "value2");
        r1.setProperties(r2Pro);


        String path = "/d1/d2/d3/d4/r1";
        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /d1/d2/d3/d4/r1");
        }

        String contentUpdated = "this is my content updated";
        r1.setContent(contentUpdated.getBytes());
        r1.setDescription("This is r1 file description updated");
        r1.setDirectory(false);

        Properties r2ProUpdate = new Properties();
        r2ProUpdate.setProperty("key1", "value1_update");
        r2ProUpdate.setProperty("key2", "value2_update");
        r2ProUpdate.setProperty("key3_update", "value3_update");
        r1.setProperties(r2ProUpdate);

        Resource r1_actual = new Resource();

        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put updated content to path /d1/d2/d3/d4/r1");
        }

        try {
            r1_actual = registry.get("/d1/d2/d3/d4/r1");
        } catch (RegistryException e) {
            fail("Couldn't get content from path /d1/d2/d3/d4/r1");
        }

//        //System.out.println("File content:" + new String((byte[]) r1.getContent()));
//        //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//        //System.out.println("Path:" + r1_actual.getResourcePath());
//        //System.out.println("File Desciption:" + r1_actual.getDescription());
//        //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//        //System.out.println("Resource Properties:" + r1_actual.getProperties());
//        //System.out.println("Resource is Directory:" + r1_actual.isDirectory());

    }

    public void testAddAnotherResource() throws Exception {
        Resource r1 = new Resource();
        String content = "this is my content2";
        r1.setContent(content.getBytes());
        r1.setDescription("r2 file description");
        String path = "/d1/d2/r2";

        Properties r2Pro = new Properties();

        r2Pro.setProperty("key1", "value1");
        r2Pro.setProperty("key2", "value2");
        r1.setProperties(r2Pro);

        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /d1/d2/r2");
        }

        Resource r1_actual = new Resource();

        try {
            r1_actual = registry.get("/d1/d2/r2");
        } catch (RegistryException e) {
            fail("Couldn't get content from path /d1/d2/r2");
        }

//         //System.out.println("File content:" + (new String((byte[]) r1.getContent())));
//         //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//         //System.out.println("Path:" + r1_actual.getResourcePath());
//         //System.out.println("File Desciption:" + r1_actual.getDescription());
//         //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//         //System.out.println("Resource Properties:" + r1_actual.getProperties());
//
    }

    public void testSetResourceDetails() throws Exception {

        Resource r1 = new Resource();
        r1.setDescription("R4 collection description");
        r1.setMediaType("jpg/xml");
        r1.setDirectory(false);
        /*r1.setParentPath("/c11/c12/c13/c14/r4");*/


        Properties r2Pro = new Properties();

        r2Pro.setProperty("key1", "value5");
        r2Pro.setProperty("key2", "value3");
        r1.setProperties(r2Pro);


        String path_collection = "/c11/c12/c13/c14/r4";

        try {
            registry.put(path_collection, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /c11/c12/c13/c14/r4");
        }

        Resource r1_actual = new Resource();

        try {
            r1_actual = registry.get("/c11/c12/c13/c14/r4");
        } catch (RegistryException e) {
            fail("Couldn't get content from path /c11/c12/c13/c14/r4");
        }

//             //System.out.println("File content get:" + new String((byte[]) r1_actual.getContent()));
        //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
        //System.out.println("Path:" + r1_actual.getPath());
        //System.out.println("File Desciption:" + r1_actual.getDescription());
        //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
        //System.out.println("Resource Properties:" + r1_actual.getProperties());
        //System.out.println("Resource MediaType:" + r1_actual.getMediaType());
        //System.out.println("Resource Properties by key:" + r1_actual.getProperty("key2"));
        //System.out.println("Resource Parent path:" + r1_actual.getParentPath());
        //System.out.println("Resource is Directory:" + r1_actual.isDirectory());
        //System.out.println("Resource get ID:" + r1_actual.getId());
        //System.out.println("Resource get CreatedTime:" + r1_actual.getCreatedTime());
        //System.out.println("Resource get LastModified Time:" + r1_actual.getLastModified());
        //System.out.println("Resource get State:" + r1_actual.getState());
        //System.out.println("Resource is content modified:" + r1_actual.isContentModified());

    }

    public void testCollectionDetails() throws Exception {
        Resource r1 = new Resource();
        String content = "this is my content4";
        r1.setContent(content.getBytes());
        r1.setDescription("r3 file description");
        String path = "/c1/c2/c3/c4/r3";

        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /c1/c2/c3/c4/r3");
        }

        Resource r1_actual = new Resource();

        /*
        try{
             r1_actual=registry.get("/c1/c2/c3");
         }catch(RegistryException e){
             fail("Couldn't get content from path /c1/c2/c3");
         }
         */

        ////System.out.println("File content:" + new String(new String((byte[]) r1.getContent())));
//         //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//         //System.out.println("Path:" + r1_actual.getResourcePath());
//         //System.out.println("File Desciption:" + r1_actual.getDescription());
//         //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//         //System.out.println("Resource Properties:" + r1_actual.getProperties());

        // Now delete the collection
        String path_delete = "/c1/c2/c3";

        try {
            registry.delete(path_delete);
        } catch (RegistryException e) {
            fail("Couldn't delete content resource " + path_delete);
        }

        boolean failed = false;
        try {
            registry.get(path_delete);
        } catch (RegistryException e) {
            failed = true;
        }

        assertTrue("Deleted resource /r1 is returned on get.", failed);
    }


    public void testSetCollectionDetails() throws Exception {

        Resource r1 = new Resource();
        r1.setDescription("C3 collection description");
        r1.setMediaType("text");
        r1.setDirectory(true);
        r1.setParentPath("/c1/c2");


        Properties r2Pro = new Properties();

        r2Pro.setProperty("key1", "value5");
        r2Pro.setProperty("key2", "value3");
        r1.setProperties(r2Pro);


        String path_collection = "/c1/c2/c3";

        try {
            registry.put(path_collection, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /c1/c2/c3");
        }

        Resource r1_actual = new Resource();

        try {
            r1_actual = registry.get("/c1/c2/c3");
        } catch (RegistryException e) {
            fail("Couldn't get content from path /c1/c2/c3");
        }

        ////System.out.println("File content:" + new String(new String((byte[]) r1.getContent())));
//         //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//         //System.out.println("Path:" + r1_actual.getResourcePath());
//         //System.out.println("File Desciption:" + r1_actual.getDescription());
//         //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//         //System.out.println("Resource Properties:" + r1_actual.getProperties());
//
        assertEquals(r1_actual.getMediaType(), r1.getMediaType());
        assertEquals(r1_actual.getParentPath(), r1.getParentPath());
//         //System.out.println("Resource Properties by key:" + r1_actual.getProperty("key2"));
        assertTrue(r1_actual.isDirectory());
//         //System.out.println("Resource get ID:" + r1_actual.getId());
//         //System.out.println("Resource get CreatedTime:" + r1_actual.getCreatedTime());
//         //System.out.println("Resource get LastModified Time:" + r1_actual.getLastModified());
//         //System.out.println("Resource get State:" + r1_actual.getState());


    }


    public void testDeleteResource() throws Exception {

        Resource r1 = new Resource();
        r1.setContent("this is file for deleting");
        r1.setDescription("this is the description of deleted file");
        r1.setMediaType("text");
        r1.setDirectory(false);


        Properties r2Pro = new Properties();
        r2Pro.setProperty("key1", "value5");
        r2Pro.setProperty("key2", "value3");
        r1.setProperties(r2Pro);

        String path = "/c11/c12/c13/r4";

        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /c11/c12/c13/r4");
        }

        String path_delete = "/c11/c12/c13/r4";

        try {
            registry.delete(path_delete);
        } catch (RegistryException e) {
            fail("Couldn't delete content resource /c11/c12/c13/r4");
        }

        boolean failed = false;
        try {
            registry.get("/c11/c12/c13/r4");
        } catch (RegistryException e) {
            failed = true;
        }

        assertTrue("Deleted resource /c11/c12/c13/r4 is returned on get.", failed);

        /*Add deleted resource again in to same path*/

        Resource r2 = new Resource();
        r2.setContent("This is new contenet added after deleting");
        r2.setDescription("this is desc for new resource");
        r2.setMediaType("text");
        r2.setDirectory(false);

        Properties r2Pro_new = new Properties();
        r2Pro_new.setProperty("key1", "value5");
        r2Pro_new.setProperty("key2", "value3");
        r2.setProperties(r2Pro_new);

        String path_new = "/c11/c12/c13/r4";

        try {
            registry.put(path_new, r2);
        } catch (RegistryException e) {
            fail("Couldn't put content to path /c11/c12/c13/r4");
        }

        Resource r1_actual = new Resource();

        try {
            r1_actual = registry.get(path_new);
        } catch (RegistryException e) {
            fail("Couldn't get content of path /c11/c12/c13/r4");
        }

        // //System.out.println("File content:" + (new String((byte[]) r1_actual.getContent())));
//        //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//        //System.out.println("Path:" + r1_actual.getResourcePath());
//        //System.out.println("File Desciption:" + r1_actual.getDescription());
//        //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//        //System.out.println("Resource Properties:" + r1_actual.getProperties());


    }

    public void testAddSpacesinResName() throws Exception {

        Resource r1 = new Resource();
        r1.setContent("this is file file content");
        r1.setDescription("this is a file name with spaces");
        r1.setMediaType("text");
        r1.setDirectory(false);


        Properties r2Pro = new Properties();
        r2Pro.setProperty("key1", "value5");
        r2Pro.setProperty("key2", "value3");
        r1.setProperties(r2Pro);

        //     String path="/d11/d12/d13/r1 space";

        /*    try {
                registry.put(path, r1);
            } catch (RegistryException e) {
                fail("Couldn't put content to path /d11/d12/d13/r1 space");
            }
        */
        Resource r1_actual = new Resource();

        /*    try {
                r1_actual=registry.get(path);
            } catch (RegistryException e) {
                fail("Couldn't get content of path /d11/d12/d13/r1 space");
            }
        */
        // //System.out.println("File content:" + (new String((byte[]) r1_actual.getContent())));
//            //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//            //System.out.println("Path:" + r1_actual.getResourcePath());
//            //System.out.println("File Desciption:" + r1_actual.getDescription());
//            //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//            //System.out.println("Resource Properties:" + r1_actual.getProperties());

    }

    public void testAddSpacesinCollName() throws Exception {

        Resource r1 = new Resource();

        r1.setDescription("this is a collection name with spaces");
        r1.setDirectory(true);


        Properties r2Pro = new Properties();
        r2Pro.setProperty("key1", "value5");
        r2Pro.setProperty("key2", "value3");
        r1.setProperties(r2Pro);

        //        String path="/col1/col2/col3 space";

        /*
                try {
                    registry.put(path, r1);
                } catch (RegistryException e) {
                    fail("Couldn't put collection /col1/col2/col3 space");
                }
        */
        Resource r1_actual = new Resource();

        /*        try {
                    r1_actual=registry.get(path);
                } catch (RegistryException e) {
                    fail("Couldn't get collecion of path /col1/col2/col3 space");
                }

        */
//                //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
//                //System.out.println("Path:" + r1_actual.getResourcePath());
//                //System.out.println("Collection Desciption:" + r1_actual.getDescription());
//                //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
//                //System.out.println("Resource Properties:" + r1_actual.getProperties());

    }
    /*
    public void testAddResourceFromURL() throws Exception{


       String path="/d25/d21/d23/r1";
       String url="http://131.107.72.15/SoapWsdl_BaseDataTypes_XmlFormatter_Service_Indigo/BaseDataTypesDocLitW.svc?wsdl";

       try{
            registry.importResource(path,url,"wsdl");

       }catch(RegistryException e) {
            fail("Couldn't import content to path:"+ path);
       }

        Resource r1_actual= new Resource();

        try{
            r1_actual=registry.get(path);
        }catch(RegistryException e){
            fail("Couldn't get content from path" + path);
        }

        boolean content=true;
        if (r1_actual==null){
            content=false;
        }

        assertTrue("Imported file is empty",content);

<<<<<<< .mine
         //System.out.println("File content:" + new String((byte[]) r1_actual.getContent()));
         //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
         //System.out.println("Path:" + r1_actual.getResourcePath());
         //System.out.println("File Desciption:" + r1_actual.getDescription());
         //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
         //System.out.println("Resource Properties:" + r1_actual.getProperties());
         //System.out.println("Resource is Directory:" + r1_actual.isDirectory());
=======
        //System.out.println("File content:" + new String((byte[]) r1_actual.getContent()));
        //System.out.println("Updated User Name:" + r1_actual.getLastUpdaterUserName());
        //System.out.println("Path:" + r1_actual.getPath());
        //System.out.println("File Desciption:" + r1_actual.getDescription());
        //System.out.println("Author Name:" + r1_actual.getAuthorUserName());
        //System.out.println("Resource Properties:" + r1_actual.getProperties());
        //System.out.println("Resource is Directory:" + r1_actual.isDirectory());
>>>>>>> .r12721

        assertEquals("LastUpdatedUser is not Equal","admin",r1_actual.getLastUpdaterUserName());

    }

    */

    public void testRenameResource() throws Exception {

        Resource r1 = new Resource();
        String content = "this is my content";
        r1.setContent(content.getBytes());
        r1.setDescription("This is r1 file description");
        r1.setDirectory(false);

        String path = "/d30/d31/r1";
        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path" + path);
        }

        Resource r1_actual = new Resource();
        try {
            r1_actual = registry.get(path);
        } catch (RegistryException e) {
            fail("Couldn't get content from path" + path);
        }

        assertEquals("Content is not equal.", new String((byte[])r1.getContent()),
                     new String((byte[])r1_actual.getContent()));

        /*rename the resource*/

        String new_path = "/d33/d34/r1";
        String new_value = null;

        try {
            new_value = registry.rename(path, new_path);
        } catch (RegistryException e) {
            fail("Can not rename the path from" + path + "to" + new_path);
        }
        //System.out.println("This is the returned value" + new_value);
    }

    public void testDeleteandUpdateResource() throws Exception {

        Resource r1 = new Resource();
        String content = "this is my content";
        r1.setContent(content.getBytes());
        r1.setDescription("This is r1 file description");
        r1.setDirectory(false);

        String path = "/d40/d43/r1";
        try {
            registry.put(path, r1);
        } catch (RegistryException e) {
            fail("Couldn't put content to path" + path);
        }

        Resource r1_actual = new Resource();
        try {
            r1_actual = registry.get(path);
        } catch (RegistryException e) {
            fail("Couldn't get content from path" + path);
        }

        assertEquals("Content is not equal.", new String((byte[])r1.getContent()),
                     new String((byte[])r1_actual.getContent()));

        boolean deleted = true;
        try {
            registry.delete(path);
        } catch (RegistryException e) {
            fail("Couldn't delete the resource from path" + path);
            deleted = false;
        }

        assertTrue("Resource not deleted", deleted);

        /*add the same resource again*/

        Resource r2 = new Resource();
        String content2 = "this is my content updated";
        r2.setContent(content2.getBytes());
        r2.setDescription("This is r1 file description");
        r2.setDirectory(false);

        String path_new = "/d40/d43/r1";
        try {
            registry.put(path_new, r2);
        } catch (RegistryException e) {
            fail("Couldn't put content to path" + path_new);
        }

        Resource r1_actual2 = new Resource();
        try {
            r1_actual2 = registry.get(path_new);
        } catch (RegistryException e) {
            fail("Couldn't get content from path" + path_new);
        }

        assertEquals("Content is not equal.", new String((byte[])r2.getContent()),
                     new String((byte[])r1_actual2.getContent()));
    }
}
