/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;

public class VersionHandlingTest extends TestCase {

    protected static Registry registry = null;

    public void setUp() {
        try {
            if (registry == null) {
                RegistryRealm registryRealm = new InMemoryRegistryRealm();
                registry = new InMemoryJDBCRegistry(registryRealm);
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testResourceRestore() throws RegistryException {
        Resource r1 = new Resource();
        r1.setContent("Content1");
        registry.put("/wso2/wsas/v1/r1/resource1", r1);
        registry.put("/wso2/wsas/v1/resource1", r1);
        Resource esb = new Resource();
        esb.setContent("ESB content");
        registry.put("/wso2/esb/v2/resource1", esb);
        Resource resource1 = registry.get("/wso2/wsas/v1");
        String[] content1 = ((String[])resource1.getContent());
        assertEquals(2, content1.length);
        assertEquals("/wso2/wsas/v1/r1", content1[0]);
        assertEquals("/wso2/wsas/v1/resource1", content1[1]);
        registry.delete("/wso2/wsas/v1");

        Resource r2 = new Resource();
        r2.setContent("Content2");
        registry.put("/wso2/wsas/v1/r2/resource2", r2);
        registry.put("/wso2/wsas/v1/resource2", r2);
        resource1 = registry.get("/wso2/wsas/v1");
        content1 = ((String[])resource1.getContent());
        assertEquals(2, content1.length);

        assertEquals("/wso2/wsas/v1/r2", content1[0]);
        assertEquals("/wso2/wsas/v1/resource2", content1[1]);


        registry.restoreVersion("/wso2/wsas?v=2");
        resource1 = registry.get("/wso2/wsas/v1");
        content1 = ((String[])resource1.getContent());
        assertEquals(2, content1.length);
        assertEquals("/wso2/wsas/v1/r1", content1[0]);
        assertEquals("/wso2/wsas/v1/resource1", content1[1]);

        Resource newresource = new Resource();
        newresource.setContent("newcontent");
        registry.put("/wso2/wsas/v1/r3", newresource);
        resource1 = registry.get("/wso2/wsas/v1");
        content1 = ((String[])resource1.getContent());
        assertEquals(3, content1.length);
        assertEquals("/wso2/wsas/v1/r1", content1[0]);
        assertEquals("/wso2/wsas/v1/resource1", content1[1]);
        assertEquals("/wso2/wsas/v1/r3", content1[2]);

        registry.restoreVersion("/wso2/wsas?v=2");
        resource1 = registry.get("/wso2/wsas/v1");
        content1 = ((String[])resource1.getContent());
        assertEquals(2, content1.length);
        assertEquals("/wso2/wsas/v1/r1", content1[0]);
        assertEquals("/wso2/wsas/v1/resource1", content1[1]);

    }

    public void testResourceVersioning() throws Exception {
        String mashupString = "testMashup";

        Resource mashup = new Resource();
        mashup.setContent(mashupString);
        registry.put("/wso2/mashup", mashup);

        Resource wsas = new Resource();
        wsas.setContent("testWSAS".getBytes());
        registry.put("/wso2/wsa", wsas);

        Resource myMashup = registry.get("/wso2/mashup");
        String content = new String((byte[])myMashup.getContent());
        assertNotNull(content);
        assertEquals(mashupString, content);
    }
}
