/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.usermanager.Realm;

public class CommentsTest extends TestCase {

     private static Registry registry = null;
    private static RegistryRealm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                realm = new InMemoryRegistryRealm();
                registry = new InMemoryJDBCRegistry(realm);
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testCommentDelete() {

        try {
            SecureRegistry adminReg =
                    new SecureRegistry(
                            RegistryConstants.ADMIN_USER,
                            RegistryConstants.ADMIN_PASSWORD,
                            registry,
                            realm);


            Realm adminRealm = adminReg.getUserRealm();
            adminRealm.getUserStoreAdmin().addUser("cu1", "cu1");

            SecureRegistry cu1Reg = new SecureRegistry("cu1", "cu1", registry, realm);

            Resource r1 = new Resource();
            r1.setContent("r1 content");
            adminReg.put("/crtest/r1", r1);

            adminRealm.getAccessControlAdmin().
                    authorizeUser("cu1", "/crtest/r1", ActionConstants.PUT);

            Resource r2 = new Resource();
            r2.setContent("r2 content");
            adminReg.put("/crtest/r2", r2);

            String c1 =
                    adminReg.addComment("/crtest/r1", new Comment("admin's comment on /crtest/r1"));

            String c2 =
                    adminReg.addComment("/crtest/r2", new Comment("admin's comment on /crtest/r2"));

            String c3 =
                    cu1Reg.addComment("/crtest/r1", new Comment("cu1's comment on /crtest/r1"));

            String c4 =
                    cu1Reg.addComment("/crtest/r2", new Comment("cu1's comment on /crtest/r2"));

            try {
                cu1Reg.delete(c1);
            } catch (RegistryException e) {
                fail("cu1 should be able to delete admin's comment on admin's resource as he has" +
                        " write prmissions on it.");
            }

            boolean failed = false;
            try {
                cu1Reg.delete(c2);
            } catch (RegistryException e) {
                failed = true;
            }
            assertTrue("cu1 should not have " +
                    "permission to delete admin's comment on admin's resource.", failed);

            try {
                cu1Reg.delete(c3);
            } catch (RegistryException e) {
                fail("cu1 should be able to delete cu1's comment on admin's resource even he does" +
                        " not have write permissions for it.");
            }

            try {
                cu1Reg.delete(c4);
            } catch (RegistryException e) {
                fail("cu1 should be able to delete cu1's comment on admin's resource. In this case" +
                        "cu1 also has write permissions on admin's resource.");
            }

        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }

    }
}
