/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.usermanager.Realm;

public class ResourceRenamingTest extends TestCase {

    private static Registry registry = null;
    private static RegistryRealm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                realm = new InMemoryRegistryRealm();
                registry = new InMemoryJDBCRegistry(realm);
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testResourceRename() {

        try {
            SecureRegistry adminReg =
                    new SecureRegistry(
                            RegistryConstants.ADMIN_USER,
                            RegistryConstants.ADMIN_PASSWORD,
                            registry,
                            realm);


            Realm adminRealm = adminReg.getUserRealm();
            adminRealm.getUserStoreAdmin().addUser("rr1", "rr1");

            SecureRegistry rr1Reg = new SecureRegistry("rr1", "rr1", registry, realm);

            Resource r1 = new Resource();
            r1.setContent("r1 content");
            adminReg.put("/rrtest/r1", r1);

            adminRealm.getAccessControlAdmin()
                    .authorizeUser("rr1", "/rrtest/r1", ActionConstants.DELETE);
            adminRealm.getAccessControlAdmin().authorizeUser("rr1", "/rrtest", ActionConstants.PUT);

            rr1Reg.rename("/rrtest/r1", "/rrtest/r2");

            Resource r11 = rr1Reg.get("/rrtest/r2");

            assertTrue("r11 does not exist.", r11 != null);

        } catch (Exception e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }
}
