/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.servlet.FileUploadUtil;
import org.wso2.registry.servlet.utils.Utils;
import org.wso2.registry.web.actions.*;
import org.wso2.registry.web.beans.AdminBean;
import org.wso2.registry.web.beans.VersionsBean;
import org.wso2.registry.web.beans.AnnouncementsBean;
import org.wso2.registry.web.utils.*;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLDecoder;

public class ControllerServlet extends HttpServlet {

    protected transient ServletConfig servletConfig;

    //To store the conetxt path
    private String contextRoot = null;

    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response)
            throws ServletException, IOException {
        initContextRoot(request);

        // if a path is not set in the session, set the path to the root
        String path = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
            request.getSession().setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
        }

        // set the user name for the session
        SecureRegistry userRegistry =
                (SecureRegistry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);
        if (userRegistry != null) {
            request.getSession().setAttribute(UIConstants.USER_ATTR, userRegistry.getUserID());

            request.getSession().setAttribute(UIConstants.IS_LOGGED_IN_ATTR, Boolean.TRUE);

            try {
                boolean isAdmin = UserUtil.isAdmin(request, userRegistry.getUserID());
                request.getSession().setAttribute(UIConstants.IS_ADMIN_ATTR, Boolean.valueOf(isAdmin));
            } catch (RegistryException e) {
                request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
                request.getRequestDispatcher(UIConstants.ERROR_JSP).forward(request, response);
                return;
            }

        } else {
            request.getSession().setAttribute(UIConstants.USER_ATTR, RegistryConstants.ANONYMOUS_USER);
            request.getSession().setAttribute(UIConstants.IS_LOGGED_IN_ATTR, Boolean.FALSE);
        }

        String uri = request.getRequestURI();
        String controlPart = uri.substring(contextRoot.length(), uri.length());

        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/signin")) {

                //SignInAction signInAction = new SignInAction();
                //signInAction.setDisplayUserName(request.getParameter("userName"));
                //signInAction.setPassword(request.getParameter("password"));
                //
                //try {
                //    signInAction.execute(request);
                //} catch (Exception e) {
                //    setErrorMessage(request, e.getMessage());
                //    e.printStackTrace();
                //}
                //
                //response.sendRedirect("/wso2registry/web" + path);

                UserUtil.signIn(request, response);

                //forwardToResources(request, response, path);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(request.getParameter(RegistryConstants.ANONYMOUS_PASSWORD));

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(
                        UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
                response.sendRedirect("/wso2registry/web");
                //forwardToResources(request, response, path);

            } else if (command.equals("/addUser")) {

                AddUserAction addUserAction = new AddUserAction();
                addUserAction.setNewUserName(request.getParameter("newUserName"));
                addUserAction.setPassword(request.getParameter("password"));
                addUserAction.setConfirmedPassword(request.getParameter("confirmedPassword"));
                addUserAction.setRoleName(request.getParameter("roleName"));
                addUserAction.setFriendlyName(request.getParameter("friendlyName"));

                try {
                    addUserAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/addRole")) {

                AddRoleAction addRoleAction = new AddRoleAction();
                addRoleAction.setNewRoleName(request.getParameter("newRoleName"));
                try {
                    addRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/applyTag")) {

                TagUtil.tag(request, response);

            } else if (command.equals("/tag")) {

                AddTagAction addTagAction = new AddTagAction();
                addTagAction.setPath(path);
                addTagAction.setTag(request.getParameter("tag"));

                try {
                    addTagAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);

            } else if (command.equals("/authorize")) {

                PermissionUtil.addUserPermission(request, response);

            } else if (command.equals("/authorize/mass")) {

                PermissionUtil.applyMassUserPermissions(request, response);

            } else if (command.equals("/authorizeRole")) {

                PermissionUtil.addRolePermission(request, response);

            } else if (command.equals("/authorizeRoles")) {

                PermissionUtil.applyMassRolePermissions(request, response);

            } else if (command.equals("/setDescription")) {

                //String resourcePath = path;
                String description = request.getParameter("description");

                SetDescriptionAction setDescriptionAction = new SetDescriptionAction();
                setDescriptionAction.setResourcePath(path);
                setDescriptionAction.setDescription(description);

                try {
                    setDescriptionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_DESCRIPTION_STRING, setDescriptionAction.getDescription());
                request.getRequestDispatcher(UIConstants.AJAX_DESCRIPTION_JSP).forward(request, response);

            } else if (command.equals("/setRating")) {

                int userRating = Integer.parseInt(request.getParameter("ratingValue"));

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(path);
                ajaxRatingAction.setUserRating(userRating);

                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

             } else if (command.equals("/renameResource")) {

                try {
                    ResourcesUtil.renameResource(request, response);
                } catch (Exception e) {
                    // todo: implement a general AJAX error segment
                    setErrorMessage(request, e.getMessage());
                    forwardToResources(request, response, path);
                }

            } else if (command.equals("/addComment")) {

                try {
                    CommentsUtil.addComment(request, response);
                } catch (Exception e) {
                    // todo: implement a general AJAX error segment
                    setErrorMessage(request, e.getMessage());
                    forwardToResources(request, response, path);
                }

            }else if (command.equals("/saveFriendlyName")) {

                try {
                    UserUtil.updateFriendlyName(request,response);
                } catch (Exception e) {
                    // todo: implement a general AJAX error segment
                    setErrorMessage(request, e.getMessage());
                    forwardToResources(request, response, path);
                }

            } else if (command.equals("/saveNewPassword")) {

                try {
                    UserUtil.updatePassword(request,response);
                } catch (Exception e) {
                    // todo: implement a general AJAX error segment
                    setErrorMessage(request, e.getMessage());
                    forwardToResources(request, response, path);
                }

            }else if (command.equals("/setProperty")) {

                PropertiesUtil.setProperty(request, response);

            } else if (command.equals("/updateProperty")) {

                PropertiesUtil.updateProperty(request, response);

            } else if (command.equals("/removeProperty")) {

                PropertiesUtil.removeProperty(request, response);

            } else if (command.equals("/addUserToRole")) {

                String userName = request.getParameter("userName");
                AddUserRoleAction addUserRoleAction = new AddUserRoleAction();
                addUserRoleAction.setUserName(userName);
                addUserRoleAction.setRoleName(request.getParameter("roleToAdd"));

                try {
                    addUserRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people/" + userName);
                //forwardToUserPage(request, response, request.getParameter("userName"));

            } else if (command.equals("/addCollection")) {

                AddCollectionAction addCollectionAction = new AddCollectionAction();
                addCollectionAction.setParentPath(request.getParameter("parentPath"));
                addCollectionAction.setCollectionName(request.getParameter("collectionName"));
                addCollectionAction.setMediaType(request.getParameter("mediaType"));
                addCollectionAction.setDescription(request.getParameter("description"));

                try {
                    addCollectionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web" + path);
                //forwardToResources(request, response, path);

            } else if (command.equals("/fetchResource")) {

                try {
                    ResourceHandler.fetchURLResource(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                }

                response.sendRedirect("/wso2registry/web" + path);

            } else if (command.equals("/addResource")) {

                handleFileUpload(request, response);

            } else if (command.equals("/addDBConfig")) {

                AdminUtil.addDBURLBasedConfiguration(request, response);

            }
        }
    }

    private void handleFileUpload(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

        String path = "";

        try {
            Registry registry = Utils.getSecureRegistry(request);

            Resource fileElement =
                    FileUploadUtil.processUpload(request);
            path = fileElement.getPath();

            registry.put(path, fileElement);

        } catch (RegistryException e) {
            e.printStackTrace();
            setErrorMessage(request, e.getMessage());
        }

        response.sendRedirect("/wso2registry/web" + getParentPath(path));
    }

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
            throws ServletException, IOException {
        String uri = URLDecoder.decode(request.getRequestURI(), "UTF-8");

        // if a path is not set in the session, set the path to the root
        String path = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
        if (path == null || path.length() == 0) {
            path = RegistryConstants.ROOT_PATH;
            request.getSession().setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
        }

        // set the user name for the session
        SecureRegistry userRegistry =
                (SecureRegistry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);
        if (userRegistry != null) {
            request.getSession().setAttribute(UIConstants.USER_ATTR, userRegistry.getUserID());
        } else {
            request.getSession().setAttribute(UIConstants.USER_ATTR, RegistryConstants.ANONYMOUS_USER);
        }

        initContextRoot(request);


        String controlPart = uri.substring(contextRoot.length());
        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH)) {

            // process request regarding resource handling

            if (uri.equals("") || uri.endsWith(UIConstants.WEB_PATH + RegistryConstants.PATH_SEPARATOR)) {
                path = RegistryConstants.ROOT_PATH;

            } else {
                path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH).
                        length(), uri.length());
            }

            // if user is browsing an old version of the resource, we append it to the path, so that
            // the backend registry gives the details of the version
            String qPart = request.getQueryString();
            if (qPart != null) {
                if (qPart.startsWith("v")) {
                    path = path + "?" + qPart;
                }
            }

            // whenever a user selects a URL for a resource path, we set that path in the user's
            // session. so the user's path is known from any part of the UI.
            request.getSession().setAttribute(UIConstants.PATH_ATTR, path);

            forwardToResources(request, response, path);

        } else if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.VERSIONS_PATH)) {

            if (uri.equals("") || uri.endsWith(UIConstants.VERSIONS_PATH + RegistryConstants.PATH_SEPARATOR)) {
                path = RegistryConstants.ROOT_PATH;

            } else {
                path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.VERSIONS_PATH).
                        length(), uri.length());
            }

            // if user is browsing an old version of the resource, we append it to the path, so that
            // the backend registry gives the details of the version
            String qPart = request.getQueryString();
            if (qPart != null && qPart.startsWith("v")) {
                path = path + "?" + qPart;
            }

            VersionsBean versionsBean ;
            try {
                versionsBean = ResourcesUtil.getVersions(request, path);
            } catch (RegistryException e) {
                request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
                request.getRequestDispatcher(UIConstants.ERROR_JSP);
                return;
            }

            request.getSession().setAttribute(UIConstants.VERSIONS_BEAN, versionsBean);
            request.getRequestDispatcher(UIConstants.VERSIONS_JSP).forward(request, response);
        } else if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.RESOURCES_PATH)) {

            if (uri.equals("") || uri.endsWith(UIConstants.RESOURCES_PATH + RegistryConstants.PATH_SEPARATOR)) {
                path = RegistryConstants.ROOT_PATH;

            } else {
                path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.RESOURCES_PATH).
                        length(), uri.length());
            }

            // if user is browsing an old version of the resource, we append it to the path, so that
            // the backend registry gives the details of the version
            String qPart = request.getQueryString();
            if (qPart != null && qPart.startsWith("v")) {
                path = path + "?" + qPart;
            }

            Resource resource ;
            try {
                resource = ResourcesUtil.getResource(request, path);
            } catch (RegistryException e) {
                request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
                request.getRequestDispatcher(UIConstants.ERROR_JSP);
                return;
            }

            if (resource == null) {
                request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, "404 Not Found");
                request.getRequestDispatcher(UIConstants.ERROR_JSP);
                return;
            }

            if (resource.isDirectory()) {
                response.sendRedirect("/wso2registry/web" + path);
            } else {
                sendResourceContent(request, response, path);
            }

        } else if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/people")) {
                forwardToUserManagement(request, response);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(RegistryConstants.ANONYMOUS_PASSWORD);

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/web");
                //forwardToResources(request, response, path);

            } else if (command.equals("/activity")) {

                forwardToActivity(request, response);

            } else if (command.equals("/admin")) {

                forwardToAdmin(request, response);

            } else if (command.equals("/activity/new")) {

                forwardToEmptyActivity(request, response);

            } else if (command.equals("/search/advanced")) {

                forwardToAdvancedSearch(request, response);

            } else if (command.equals("/search/advanced/new")) {

                forwardToNewAdvancedSearch(request, response);

            } else if (command.startsWith("/people") && command.length() > "/people/.".length()) {

                String userName = uri.substring("/wso2registry/system/people/".length(), uri.length());

                forwardToUserPage(request, response, userName);

            }else if(command.equals("/about")){
                request.getRequestDispatcher(UIConstants.ABOUT_JSP).forward(request, response);
            }

            else if (command.equals("/removeUserFromRole")) {

                RemoveUserRoleAction removeUserRoleAction = new RemoveUserRoleAction();
                removeUserRoleAction.setUserName(request.getParameter("user"));
                removeUserRoleAction.setRoleName(request.getParameter("role"));

                try {
                    removeUserRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people/" + request.getParameter("user"));
                //forwardToUserPage(request, response, request.getParameter("user"));

            } else if (command.equals("/removeUser")) {

                RemoveUserAction removeUserAction = new RemoveUserAction();
                removeUserAction.setUserName(request.getParameter("user"));

                try {
                    removeUserAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/removeRole")) {

                RemoveRoleAction removeRoleAction = new RemoveRoleAction();
                removeRoleAction.setRoleName(request.getParameter("role"));

                try {
                    removeRoleAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                response.sendRedirect("/wso2registry/system/people");
                //forwardToUserManagement(request, response);

            } else if (command.equals("/search")) {

                forwardToSearch(request, response);

            } else if (command.equals("/getTagCloud")) {

                try {
                    TagUtil.sendTagCloud(request, response, null);
                    return;
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                }

                forwardToResources(request, response, path);

            } else if (command.equals("/getInitialRating")) {

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(request.getParameter("resourcePath"));
                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

            } else if (command.equals("/getRating")) {

                String rating = request.getParameter("userRating");
                int iRating = Integer.parseInt(rating);

                AjaxRatingAction ajaxRatingAction = new AjaxRatingAction();
                ajaxRatingAction.setResourcePath(request.getParameter("resourcePath"));
                ajaxRatingAction.setUserRating(iRating);
                try {
                    ajaxRatingAction.execute(request);
                } catch (RegistryException e) {
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_RATING_BEAN, ajaxRatingAction);
                request.getRequestDispatcher(UIConstants.AJAX_RATING_JSP).forward(request, response);

            } else if (command.equals("/getDescription")) {

                GetDescriptionAction getDescriptionAction = new GetDescriptionAction();
                getDescriptionAction.setResourcePath(request.getParameter("resourcePath"));

                try {
                    getDescriptionAction.execute(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    e.printStackTrace();
                }

                request.getSession().setAttribute(UIConstants.AJAX_DESCRIPTION_STRING, getDescriptionAction.getDescription());
                request.getRequestDispatcher(UIConstants.AJAX_DESCRIPTION_JSP).forward(request, response);

            } else if (command.equals("/deleteResource")) {

                DeleteResourceAction deleteResourceAction = new DeleteResourceAction();
                deleteResourceAction.setPathToDelete(request.getParameter("resourcePath"));

                try {
                    deleteResourceAction.execute(request);
                } catch (Exception e) {
                    setErrorMessage(request, e.getMessage());
                }

                response.sendRedirect("/wso2registry/web" + path);

            } else if (command.equals("/getResourceMediaTypes")) {

                String mediaTypes ;
                try {
                    mediaTypes = MediaTypesReader.getResourceMediaTypeMappings(request);
                } catch (RegistryException e) {
                    mediaTypes = "txt:text,wsdl:wsdl/xml,xsd:xsd/xml,iml:idea/proj,ipr:idea/projs";
                }

                response.getWriter().write(mediaTypes);
            } else if (command.equals("/removeDBConfig")) {

                AdminUtil.removeDBConfiguration(request, response);

            } else if (command.equals("/applyConfig")) {

                try {
                    AdminUtil.applyDBConfiguration(request, response);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    forwardToAdmin(request, response);
                }

                response.sendRedirect("/wso2registry/web/");

            } else if (command.equals("/restore")) {

                try {
                    ResourcesUtil.restoreVersion(request);
                } catch (RegistryException e) {
                    setErrorMessage(request, e.getMessage());
                    forwardToResources(request, response, path);
                }

                response.sendRedirect("/wso2registry/web" + path);
            }

        }
    }

    private void sendResourceContent(HttpServletRequest request, HttpServletResponse response, String path) {

        ResourceHandlerAction handlerAction = new ResourceHandlerAction();

        Resource resource = null;
        try {
            resource = handlerAction.getResource(request, path);
        } catch (RegistryException e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        try {
            Object content = null;
            ServletOutputStream servletOutputStream = response.getOutputStream();
            if (resource != null) {
                content = resource.getContent();
            }
            if (content != null) {

                if (resource.getMediaType() != null && resource.getMediaType().length() > 0) {
                    response.setContentType(resource.getMediaType());
                } else {
                    response.setContentType("application/download");
                }

                if (content instanceof byte[]) {
                    servletOutputStream.write((byte[]) content);
                    response.flushBuffer();
                } else {
                    response.getWriter().write(content.toString());
                }
            }

            servletOutputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
//handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
    }

    private String getParentPath(String resourcePath) {

        String parentPath;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = null;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        return parentPath;
    }

    private void forwardToResources(HttpServletRequest request, HttpServletResponse response, String path) {

        try {
            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(path);
            details.execute(request);

            CollectionViewAction collection = null;
            if (details.isCollection()) {
                collection = new CollectionViewAction();
                collection.setPath(path);
                collection.execute(request);
            }

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
            request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

        } catch (AuthorizationFailedException e) {

            request.getSession().setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);
            request.getSession().setAttribute(UIConstants.ERROR_CODE, "401 Unauthorized");
            request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());

            try {
                request.getRequestDispatcher(UIConstants.ERROR_JSP).forward(request, response);
            } catch (ServletException e1) {
                e1.printStackTrace();
            } catch (IOException e1) {
                e1.printStackTrace();
            }

            return;

        } catch(RegistryException re) {

            setErrorMessage(request, re.getMessage());
            re.printStackTrace();
        }

        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    private void forwardToUserManagement(HttpServletRequest request, HttpServletResponse response) {

        UserManagementAction userManagementAction = new UserManagementAction();
        try {
            userManagementAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.USER_MANAGEMENT_BEAN, userManagementAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.USER_MANAGEMENT_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToAdmin(HttpServletRequest request, HttpServletResponse response) {

        AdminBean adminBean = AdminUtil.getAdminBean(request);
        request.getSession().setAttribute(UIConstants.ADMIN_BEAN, adminBean);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.ADMIN_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivityAction = new RecentActivityAction();
        recentActivityAction.setUserName(request.getParameter("userName"));
        recentActivityAction.setResourcePath(request.getParameter("path"));
        recentActivityAction.setFromDate(request.getParameter("fromDate"));
        recentActivityAction.setToDate(request.getParameter("toDate"));
        recentActivityAction.setFilter(request.getParameter("filter"));

        try {
            recentActivityAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivityAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToEmptyActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivity = new RecentActivityAction();
        recentActivity.setRequest(request);
        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivity);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToNewAdvancedSearch(HttpServletRequest request, HttpServletResponse response) {

        AdvancedSearchAction advancedSearchAction = new AdvancedSearchAction();
        advancedSearchAction.setRequest(request);
        request.getSession().setAttribute(UIConstants.ADVANCED_SEARCH_BEAN, advancedSearchAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.ADVANCED_SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToAdvancedSearch(HttpServletRequest request, HttpServletResponse response) {

        AdvancedSearchAction advancedSearchAction = new AdvancedSearchAction();
        advancedSearchAction.setResourcePath(request.getParameter("resourcePath"));
        advancedSearchAction.setAuthorName(request.getParameter("author"));
        advancedSearchAction.setUpdaterName(request.getParameter("updater"));
        advancedSearchAction.setCreatedAfter(request.getParameter("createdAfter"));
        advancedSearchAction.setCreatedBefore(request.getParameter("createdBefore"));
        advancedSearchAction.setUpdatedAfter(request.getParameter("updatedAfter"));
        advancedSearchAction.setUpdatedBefore(request.getParameter("updatedBefore"));
        advancedSearchAction.setTags(request.getParameter("tags"));
        advancedSearchAction.setCommentWords(request.getParameter("commentWords"));

        try {
            advancedSearchAction.execute(request);
        } catch (RegistryException e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.ADVANCED_SEARCH_BEAN, advancedSearchAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.ADVANCED_SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToSearch(HttpServletRequest request, HttpServletResponse response) {

        SearchAction searchAction = new SearchAction();
        searchAction.setTag(request.getParameter("criteria"));

        try {
            searchAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.SEARCH_BEAN, searchAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.SEARCH_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToUserPage(HttpServletRequest request, HttpServletResponse response, String userName) {
        UserDetailsAction userDetailsAction = new UserDetailsAction();
        userDetailsAction.setDisplayUserName(userName);

        try {
            userDetailsAction.execute(request);
        } catch (Exception e) {
            setErrorMessage(request, e.getMessage());
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.USER_BEAN, userDetailsAction);
        setAnnouncementsBean(request);

        try {
            request.getRequestDispatcher(UIConstants.USER_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void setAnnouncementsBean(HttpServletRequest request) {
        AnnouncementsBean announcementsBean = AnnouncementsUtil.getAnnouncementsBean();
        request.getSession().setAttribute(UIConstants.ANNOUNCEMENTS_BEAN, announcementsBean);
    }

    private void setErrorMessage(HttpServletRequest request, String message) {
        request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, message);
    }
}
