/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;

import javax.servlet.http.HttpServletRequest;
import java.util.Properties;
import java.util.Map;
import java.util.HashMap;

public class AddUserAction extends AbstractRegistryAction {

    private String newUserName;
    private String password;
    private String confirmedPassword;
    private String roleName;
    private String friendlyName;

    public String execute(HttpServletRequest request) throws Exception {

        setRequest(request);
        
        if (newUserName == null || password == null || confirmedPassword == null) {
            return ERROR;
        }

        if (!password.equals(confirmedPassword)) {
            return ERROR;
        }

        SecureRegistry secureRegistry = (SecureRegistry) getRegistry();
        Realm userRealm = secureRegistry.getUserRealm();

        userRealm.getUserStoreAdmin().addUser(newUserName, password);

        Map currentProperties = userRealm.getUserStoreAdmin().getUserProperties(newUserName);
        currentProperties.put("friendlyName", friendlyName);
        userRealm.getUserStoreAdmin().setUserProperties(newUserName, currentProperties);

        if (roleName != null && !roleName.equalsIgnoreCase("1")) {
            userRealm.getUserStoreAdmin().addUserToRole(newUserName, roleName);
        }

        return SUCCESS;
    }

    public String getNewUserName() {
        return newUserName;
    }

    public void setNewUserName(String newUserName) {
        this.newUserName = newUserName;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getConfirmedPassword() {
        return confirmedPassword;
    }

    public void setConfirmedPassword(String confirmedPassword) {
        this.confirmedPassword = confirmedPassword;
    }

    public String getRoleName() {
        return roleName;
    }

    public void setRoleName(String roleName) {
        this.roleName = roleName;
    }

    public void setFriendlyName(String friendlyName) {
        this.friendlyName = friendlyName;
    }
}
