/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.RegistryException;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

import javax.servlet.http.HttpServletRequest;

public class RemoveUserAction extends AbstractRegistryAction {

    private String userName;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);
        
        if (userName == null) {
            return ERROR;
        }
        
        SecureRegistry secureRegistry = (SecureRegistry) getRegistry();
        Realm userRealm = secureRegistry.getUserRealm();

        try {
            userRealm.getUserStoreAdmin().deleteUser(userName);
        } catch (UserManagerException e) {
            String msg = "Could not remove the user: " + userName + ". Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return SUCCESS;
    }

    public String getNewUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }
}
