/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.RegistryException;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

import javax.servlet.http.HttpServletRequest;

public class RemoveUserRoleAction extends AbstractRegistryAction {

    private String userName;
    private String roleName;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);
        
        SecureRegistry registry = (SecureRegistry) getRegistry();
        Realm realm = registry.getUserRealm();

        try {
            realm.getUserStoreAdmin().removeUserFromRole(userName, roleName);
        } catch (UserManagerException e) {
            String msg = "Could not remove the user " + userName + " from the role " + roleName + ". Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        return SUCCESS;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getRoleName() {
        return roleName;
    }

    public void setRoleName(String roleName) {
        this.roleName = roleName;
    }
}
