/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.Registry;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.secure.SecureRegistry;
import org.wso2.registry.web.ConsoleConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

public class ResourceHandler {

    public static void fetchURLResource(HttpServletRequest request) throws RegistryException {

        String parentPath = request.getParameter("path");
        String resourceName = request.getParameter("filename");

        String resourcePath;
        if (RegistryConstants.ROOT_PATH.equals(parentPath)) {
            resourcePath = RegistryConstants.ROOT_PATH + resourceName;
        } else {
            resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
        }

        String fetchURL = request.getParameter("fetchURL");
        String mediaType = request.getParameter("mediaType");
        String description = request.getParameter("description");
        Resource metadata = new Resource();
        metadata.setMediaType(mediaType);
        metadata.setDescription(description);

        SecureRegistry userRegistry = getUserRegistry(request);
        userRegistry.importResource(resourcePath, fetchURL, metadata);
    }

    private static SecureRegistry getUserRegistry(HttpServletRequest request)
            throws RegistryException {

        SecureRegistry userRegistry =
                (SecureRegistry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);

        if (userRegistry == null) {

            // user is not logged in. create a annoymous userRegistry for the user.

            ServletContext context =
                    request.getSession().getServletContext();

            Registry jdbcRegistry =
                    (Registry) context.getAttribute(RegistryConstants.REGISTRY);

            RegistryRealm realm = (RegistryRealm) request.getSession().getServletContext().
                    getAttribute(RegistryConstants.REGISTRY_REALM);

            SecureRegistry secureRegistry = new SecureRegistry(
                    RegistryConstants.ANONYMOUS_USER, "guest", jdbcRegistry, realm);

            request.getSession().setAttribute(ConsoleConstants.USER_REGISTRY, secureRegistry);

            userRegistry = secureRegistry;
        }

        return userRegistry;
    }
}
