/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.utils;

import org.wso2.registry.secure.SecureRegistry;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Registry;
import org.wso2.registry.web.beans.VersionsBean;
import org.wso2.registry.web.actions.utils.VersionPath;
import org.wso2.registry.web.actions.ResourceDetailsAction;
import org.wso2.registry.web.actions.CollectionViewAction;
import org.wso2.registry.web.UIConstants;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;

public class ResourcesUtil {

    public static Resource getResource(HttpServletRequest request, String path)
            throws RegistryException {

        SecureRegistry secureRegistry = CommonUtil.getUserRegistry(request);

        String resourcePath = path;
        if (path.indexOf("?") > 0) {
            resourcePath = path.substring(0, path.indexOf("?"));
        }

        if (secureRegistry.resourceExists(resourcePath)) {
            return secureRegistry.get(path);
        } else {
            return null;
        }
    }

    public static VersionsBean getVersions(HttpServletRequest request, String path)
            throws RegistryException {

        SecureRegistry secureRegistry = CommonUtil.getUserRegistry(request);

        VersionsBean versionsBean = new VersionsBean();
        versionsBean.setResourcePath(path);

        String[] versions = secureRegistry.getVersions(path);
        for (int i = 0; i < versions.length; i++) {
            VersionPath versionPath = new VersionPath();
            versionPath.setCompleteVersionPath(versions[i]);
            versionPath.setActiveResourcePath(path);

            if (versions[i].indexOf("?v=") > 0) {
                long versionNumber = Long.parseLong(versions[i].split("\\?v=")[1]);
                versionPath.setVersionNumber(versionNumber);
            }

            Resource versionResource = secureRegistry.get(versions[i]);
            if (versionResource != null) {
                versionPath.setUpdater(versionResource.getLastUpdaterUserName());
                versionPath.setUpdatedOn(versionResource.getLastModified());
            }

            versionsBean.getVersionPaths().add(versionPath);
        }

        return versionsBean;
    }

    public static void restoreVersion(HttpServletRequest request) throws RegistryException {

        SecureRegistry secureRegistry = CommonUtil.getUserRegistry(request);
        String versionPath = request.getParameter("versionPath");
        secureRegistry.restoreVersion(versionPath);
    }

    public static void renameResource(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String parentPath = request.getParameter("parentPath");
        if (!parentPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            parentPath = parentPath + RegistryConstants.PATH_SEPARATOR;
        }

        String oldResourcePath = request.getParameter("oldResourcePath");
        String newName = request.getParameter("newName");

        String newResourcePath;
        if (newName.startsWith(RegistryConstants.ROOT_PATH)) {
            newResourcePath = newName;
        } else {
            newResourcePath = parentPath + newName;
        }

        ResourceDetailsAction resourceAction = null;
        CollectionViewAction collectionView = null;
        try {
            Registry registry = CommonUtil.getUserRegistry(request);
            registry.rename(oldResourcePath, newResourcePath);

        } catch (RegistryException e) {
            String msg = "Failed to rename the resource: " + oldResourcePath + " to name: " +
                    newResourcePath + ". Caused by: " + e.getMessage();
            setErrorMessage(request, msg);
        }

        try {
            resourceAction = new ResourceDetailsAction();
            resourceAction.setPath(parentPath);
            resourceAction.execute(request);

            collectionView = new CollectionViewAction();
            collectionView.setPath(parentPath);
            collectionView.execute(request);
        } catch (RegistryException e) {
            String msg = "Failed to get details of the collection: " +
                    parentPath + ". Caused by: " + e.getMessage();
            setErrorMessage(request, msg);
        }

        request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, resourceAction);
        request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collectionView);

        request.getRequestDispatcher(UIConstants.AJAX_ENTRY_LIST_JSP).forward(request, response);
    }

    private static void setErrorMessage(HttpServletRequest request, String message) {
        request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, message);
    }
}
